@@ -2,8 +2,8 @@
 
 ;; Author: Jimmy Yuen Ho Wong <wyuenho@gmail.com>
 ;; Maintainer: Jimmy Yuen Ho Wong <wyuenho@gmail.com>
-;; Package-Version: 20250803.1832
-;; Package-Revision: 6b62cf5f7e96
+;; Package-Version: 20251217.2147
+;; Package-Revision: 222f1da89246
 ;; Package-Requires: ((emacs "27.1") (f "0.6.0") (map "3.3.1") (seq "2.24"))
 ;; Homepage: https://github.com/wyuenho/emacs-pet/
 ;; Keywords: tools
@@ -784,6 +784,10 @@ Return nil if the file is not found." file-name)))
   :file-name "pixi.toml"
   :parser pet-parse-config-file)
 
+(pet-def-config-accessor hatch
+  :file-name "hatch.toml"
+  :parser pet-parse-config-file)
+
 (defun pet-use-pre-commit-p ()
   "Whether the current project is using `pre-commit'.
 
@@ -845,6 +849,15 @@ Returns the path to the `pipenv' executable."
   (and (pet-pipfile)
        (pet--executable-find "pipenv" t)))
 
+(defun pet-use-hatch-p ()
+  "Whether the current project is using `hatch'.
+
+Returns the path to the `hatch' executable."
+  (and (or (pet-hatch)
+           (let-alist (pet-pyproject)
+             .tool.hatch.envs))
+       (pet--executable-find "hatch" t)))
+
 (defun pet-pre-commit-config-has-hook-p (id)
   "Determine if the `pre-commit' configuration has a hook.
 
@@ -1038,33 +1051,60 @@ Selects a virtualenv in the following order:
 
 1. Cached virtualenv path (from previous detection or manual switching).
 2. The value of the environment variable `VIRTUAL_ENV' if defined.
-3. Poetry virtualenv from `pyproject.toml'.
-4. Pipenv virtualenv from `Pipfile'.
-5. A directory in `pet-venv-dir-names' in the project root if found.
-6. Pyenv virtualenv from `.python-version'."
-  (let ((root (pet-project-root)))
-    (or (pet-cache-get (list root :virtualenv))
-        (let ((venv-path
-               (cond ((when-let* ((ev (getenv "VIRTUAL_ENV")))
-                        (expand-file-name ev)))
-                     ((when-let* ((program (pet-use-poetry-p))
-                                  (default-directory (file-name-directory (pet-pyproject-path))))
-                        (pet-run-process-get-output program "env" "info" "--no-ansi" "--path")))
-                     ((when-let* ((program (pet-use-pipenv-p))
-                                  (default-directory (file-name-directory (pet-pipfile-path))))
-                        (pet-run-process-get-output program "--quiet" "--venv")))
-                     ((when-let* ((dir (cl-loop for name in pet-venv-dir-names
-                                                with dir = nil
-                                                if (setq dir (locate-dominating-file default-directory name))
-                                                return (file-name-as-directory (concat dir name)))))
-                        (expand-file-name dir)))
-                     ((when-let* ((program (pet-use-pyenv-p))
-                                  (default-directory (file-name-directory (pet-python-version-path))))
-                        (file-truename (pet-run-process-get-output program "prefix")))))))
-          ;; root maybe nil when not in a project, this avoids caching a nil
-          (when root
-            (pet-cache-put (list root :virtualenv) venv-path))
-          venv-path))))
+3. Pixi default environment from `pixi.toml' or `pyproject.toml'.
+4. Conda/Mamba environment from `environment*.yml'.
+5. Poetry virtualenv from `pyproject.toml'.
+6. Hatch virtualenv from `hatch.toml' or `pyproject.toml'.
+7. Pipenv virtualenv from `Pipfile'.
+8. A directory in `pet-venv-dir-names' in the project root if found.
+9. Pyenv virtualenv from `.python-version'."
+  (let* ((root (pet-project-root))
+         (cached (pet-cache-get (list root :virtualenv))))
+    (cond
+     ;; Cache hit with 'none - no virtualenv exists, return nil
+     ((eq cached 'none) nil)
+     ;; Cache hit with actual virtualenv path
+     (cached cached)
+     ;; Cache miss - run detection
+     (t
+      (let ((venv-path
+             (cond ((when-let* ((ev (getenv "VIRTUAL_ENV")))
+                      (expand-file-name ev)))
+                   ((when-let* ((program (pet-use-pixi-p))
+                                (result (pet-run-process-get-output program "info" "--json"))
+                                (info (pet-parse-json result)))
+                      (let-alist info
+                        .environments_info.default.prefix)))
+                   ((when-let* ((program (or (pet-use-conda-p) (pet-use-mamba-p)))
+                                (env-config (pet-environment)))
+                      (let-alist env-config
+                        .prefix)))
+                   ((when-let* ((program (pet-use-poetry-p))
+                                (default-directory (file-name-directory (pet-pyproject-path))))
+                      (pet-run-process-get-output program "env" "info" "--no-ansi" "--path")))
+                   ((when-let* ((program (pet-use-hatch-p))
+                                (config-file (or (pet-hatch-path) (pet-pyproject-path)))
+                                (default-directory (file-name-directory config-file)))
+                      (pet-run-process-get-output program "env" "find" "default")))
+                   ((when-let* ((program (pet-use-pipenv-p))
+                                (default-directory (file-name-directory (pet-pipfile-path))))
+                      (pet-run-process-get-output program "--quiet" "--venv")))
+                   ((when-let* ((dir (cl-loop for name in pet-venv-dir-names
+                                              with dir = nil
+                                              if (setq dir (locate-dominating-file default-directory name))
+                                              return (file-name-as-directory (concat dir name)))))
+                      (expand-file-name dir)))
+                   ((when-let* ((program (pet-use-pyenv-p))
+                                (default-directory (file-name-directory (pet-python-version-path))))
+                      (file-truename (pet-run-process-get-output program "prefix")))))))
+        ;; FIXME: when no virtualenv is found, pet-mode should confirm with the
+        ;; user whether he would like to install his virtualenvs first, ignore
+        ;; for now, or ignore and don't ask again for this project.
+
+        ;; root maybe nil when not in a project, this avoids caching a nil key
+        (when root
+          (pet-cache-put (list root :virtualenv) (or venv-path 'none)))
+        venv-path)))))
 
 (cl-defmacro pet-def-env-list (name &key args parse-output)
   "Define an environment listing function.
@@ -1110,6 +1150,13 @@ environments.  This expression must return a list of strings."
   :parse-output
   (let-alist (pet-parse-json output) .envs))
 
+(pet-def-env-list hatch
+  :args ("env" "show" "--json")
+  :parse-output
+  (mapcar (lambda (env-name)
+            (pet-run-process-get-output program "env" "find" env-name))
+          (mapcar #'symbol-name (mapcar #'car (pet-parse-json output)))))
+
 (cl-defmacro pet-def-env-switch (name &key env-list-fn prompt-text)
   "Define a environment switching function.
 
@@ -1160,6 +1207,10 @@ returned by `%s'." name name env-list-fn)))
   :env-list-fn pet-mamba-environments
   :prompt-text "Please select a mamba environment: ")
 
+(pet-def-env-switch hatch
+  :env-list-fn pet-hatch-environments
+  :prompt-text "Please select a hatch environment: ")
+
 
 
 (defvar flycheck-mode)
@@ -1585,6 +1636,7 @@ buffer local values."
 
     (setq-local lsp-jedi-executable-command
                 (pet-executable-find "jedi-language-server"))
+    (setq-local lsp-jedi-workspace-environment-path python)
     (setq-local lsp-pyls-plugins-jedi-environment python-shell-virtualenv-root)
     (setq-local lsp-pyls-server-command (list (pet-executable-find "pyls")))
     (setq-local lsp-pylsp-plugins-jedi-environment python-shell-virtualenv-root)
@@ -1638,6 +1690,7 @@ buffer local values."
   (pet-flycheck-teardown)
 
   (kill-local-variable 'lsp-jedi-executable-command)
+  (kill-local-variable 'lsp-jedi-workspace-environment-path)
   (kill-local-variable 'lsp-pyls-plugins-jedi-environment)
   (kill-local-variable 'lsp-pyls-server-command)
   (kill-local-variable 'lsp-pylsp-plugins-jedi-environment)
@@ -1697,6 +1750,7 @@ has assigned to."
                        flycheck-python-pycompile-executable
                        flycheck-python-ruff-executable
                        lsp-jedi-executable-command
+                       lsp-jedi-workspace-environment-path
                        lsp-pyls-plugins-jedi-environment
                        lsp-pyls-server-command
                        lsp-pylsp-plugins-jedi-environment
